"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var CodeMirror = require("codemirror");
require("../style/index.css");
var keyword_control_define = new RegExp("(^\\s*)(\\bdefine\\b)");
var define_class = new RegExp("([\\t ]+[\\w-]+\\b)");
var define_nametag = new RegExp("([\\t ]+[\\w-]+\\b)");
var keyword_control_end = new RegExp("(\\bend\\b)");
var keyword_control_newline = new RegExp("(\\.{3})");
var support_function_an = new RegExp("\\s+(an?)\\b");
var support_function_auxiliary_verb = new RegExp("\\b(shall|must|should|could|will)\\b");
var keyword_control_environments = new RegExp("\\b(world|components?|variables?|parameters?|comments?|needs?|relations?|arguments?|subclauses?|(goal|transformation|design)-(requirement|constraint))\\b");
var support_function_isbe = new RegExp("\\b(is|be)\\b");
var entity_name_tag_label = new RegExp("\\s*[\\w-]+:");
var support_function_logical = new RegExp("\\b(at least)\\b"); //\\b(at([ \\t]+|\\.{3}[\\s]*)+(most|least)|(smaller|greater)([ \\t]+|\\.{3}[\\s]*)+than|(not([ \\t]+|\\.{3}[\\s]*)+)?equal([ \\t]+|\\.{3}[\\s]*)+to|or|and)\\b
var constant_language_types = new RegExp("\\b(real|string)\\b");
var keyword_control_with = new RegExp("\\bwith[\\t ]+(subclauses?|arguments?)\\b");
const ESL_MIME_TYPE = "text/x-elephant";
const ESL_EXTENSION = ".esl";
const ESL_NAME = "Elephant";
function esl_mode() {
    return {
        startState: function () {
            return {
                define: 0
            };
        },
        token: function (stream, state) {
            if (stream.peek() === "#") {
                stream.skipToEnd();
                return "comment";
            }
            // Reset states on newline, but keep once if ... matches.
            if (stream.match(keyword_control_newline)) {
                return "keyword";
            }
            // Define block "define"
            if (state.define === 0) {
                if (stream.match(keyword_control_define)) {
                    state.define = 1;
                    return "keyword";
                }
            }
            // Define block "class" (component, variable, etc.)
            else if (state.define === 1) {
                if (stream.match(define_class)) {
                    state.define = 2;
                    return "type";
                }
                else {
                    state.define = 0;
                }
            }
            // Define block component nametag
            else if (state.define === 2) {
                state.define = 0;
                if (stream.match(define_nametag)) {
                    return "tag";
                }
            }
            // Simple regexes
            if (stream.match(keyword_control_end)) {
                return "keyword";
            }
            else if (stream.match(support_function_an)) {
                return "keyword";
            }
            else if (stream.match(support_function_auxiliary_verb)) {
                return "keyword";
            }
            else if (stream.match(keyword_control_environments)) {
                return "keyword";
            }
            else if (stream.match(support_function_isbe)) {
                return "operator";
            }
            else if (stream.match(entity_name_tag_label)) {
                return "tag";
            }
            else if (stream.match(support_function_logical)) {
                return "operator";
            }
            else if (stream.match(constant_language_types)) {
                return "meta";
            }
            else if (stream.match(keyword_control_with)) {
                return "keyword";
            }
            // Nothing matched, reset state and eat some chars for breakfast.
            state.define = 0;
            if (!stream.eatSpace()) {
                stream.next();
            }
            return null;
        }
    };
}
const extension = {
    id: "esl-jupyterlab-codemirror",
    autoStart: true,
    activate: (app) => {
        console.log("JupyterLab extension esl-jupyterlab-codemirror is being activated...");
        CodeMirror.defineMode(ESL_NAME.toLowerCase(), esl_mode);
        CodeMirror.defineMIME(ESL_MIME_TYPE, { name: ESL_NAME.toLowerCase() });
        CodeMirror.modeInfo.push({
            ext: [ESL_EXTENSION.substring(1)],
            mime: ESL_MIME_TYPE,
            mode: ESL_NAME.toLowerCase(),
            name: ESL_NAME
        });
        console.log("Associating .esl format with newly added mode...");
        app.docRegistry.addFileType({
            name: ESL_NAME,
            displayName: "Elephant Specification language File",
            extensions: [ESL_EXTENSION],
            mimeTypes: [ESL_MIME_TYPE],
            iconClass: "jp-MaterialIcon jp-PasteIcon"
        });
        console.log("Done!");
    }
};
exports.default = extension;
